import * as React from 'react';
import { Notify } from 'rsuite-notification';
import _ from 'lodash';
import Icon from '../Icon';
import { STATUS_ICON_NAMES } from '../constants';
import { getClassNamePrefix } from '../utils/prefix';
var classPrefix = getClassNamePrefix() + "notification";
var defaultOptions = {
  classPrefix: classPrefix
};
Notify.config(defaultOptions);

function appendIcon(type, content) {
  if (!STATUS_ICON_NAMES[type]) {
    return content;
  }

  return React.createElement("div", {
    className: classPrefix + "-title-with-icon"
  }, React.createElement(Icon, {
    icon: STATUS_ICON_NAMES[type]
  }), content);
}

function proxy(type, config) {
  config.title = appendIcon(type, config.title);
  Notify[type](config);
}

var sendMessage = _.curry(proxy);

export default {
  open: sendMessage('open'),
  info: sendMessage('info'),
  success: sendMessage('success'),
  warning: sendMessage('warning'),
  error: sendMessage('error'),
  remove: function remove(key) {
    Notify.remove(key);
  }
};